@@ -1,4 +1,4 @@
-/* $Id: files.c 5519 2015-12-31 19:20:40Z bens $ */
+/* $Id: files.c 5629 2016-02-10 20:29:23Z bens $ */
 /**************************************************************************
  *   files.c                                                              *
  *                                                                        *
@@ -33,6 +33,40 @@
 #include <pwd.h>
 #include <libgen.h>
 
+/* Verify that the containing directory of the given filename exists. */
+bool has_valid_path(const char *filename)
+{
+    char *parentdir;
+    struct stat parentinfo;
+    bool validity = FALSE;
+
+    if (strrchr(filename, '/') == NULL)
+	parentdir = mallocstrcpy(NULL, ".");
+    else
+	parentdir = dirname(mallocstrcpy(NULL, filename));
+
+    if (stat(parentdir, &parentinfo) == -1) {
+	if (errno == ENOENT)
+	    statusbar(_("Directory '%s' does not exist"), parentdir);
+	else
+	    statusbar(_("Path '%s': %s"), parentdir, strerror(errno));
+    } else if (!S_ISDIR(parentinfo.st_mode)) {
+	statusbar(_("Path '%s' is not a directory"), parentdir);
+    } else {
+	if (access(parentdir, X_OK) == -1)
+	    statusbar(_("Path '%s' is not accessible"), parentdir);
+	else
+	    validity = TRUE;
+    }
+
+    free(parentdir);
+
+    if (!validity)
+	beep();
+
+    return validity;
+}
+
 /* Add an entry to the openfile openfilestruct.  This should only be
  * called from open_buffer(). */
 void make_new_buffer(void)
@@ -101,6 +135,32 @@ void initialize_buffer_text(void)
     openfile->totsize = 0;
 }
 
+/* Mark the current file as modified if it isn't already, and then
+ * update the titlebar to display the file's new status. */
+void set_modified(void)
+{
+    if (openfile->modified)
+	return;
+
+    openfile->modified = TRUE;
+    titlebar(NULL);
+
+#ifndef NANO_TINY
+    if (!ISSET(LOCKING) || openfile->filename[0] == '\0')
+	return;
+
+    if (openfile->lock_filename == NULL) {
+	/* TRANSLATORS: Try to keep this at most 76 characters. */
+	statusbar(_("Warning: Modifying a file which is not locked,"
+			" check directory permission?"));
+    } else {
+	char *fullname = get_full_path(openfile->filename);
+	write_lockfile(openfile->lock_filename, fullname, TRUE);
+	free(fullname);
+    }
+#endif
+}
+
 #ifndef NANO_TINY
 /* Actually write the lockfile.  This function will ALWAYS annihilate
  * any previous version of the file.  We'll borrow INSECURE_BACKUP here
@@ -256,10 +316,9 @@ int do_lockfile(const char *filename)
     size_t locknamesize = strlen(filename) + strlen(locking_prefix)
 		+ strlen(locking_suffix) + 3;
     char *lockfilename = charalloc(locknamesize);
-    char *lockfiledir = NULL;
     static char lockprog[11], lockuser[17];
     struct stat fileinfo;
-    int lockfd, lockpid;
+    int lockfd, lockpid, retval = -1;
 
     snprintf(lockfilename, locknamesize, "%s/%s%s%s", dirname(namecopy1),
 		locking_prefix, basename(namecopy2), locking_suffix);
@@ -271,14 +330,16 @@ int do_lockfile(const char *filename)
     if (stat(lockfilename, &fileinfo) != -1) {
 	ssize_t readtot = 0;
 	ssize_t readamt = 0;
-	char *lockbuf = charalloc(8192);
-	char *promptstr = charalloc(128);
+	char *lockbuf, *promptstr;
 	int ans;
+
 	if ((lockfd = open(lockfilename, O_RDONLY)) < 0) {
 	    statusbar(_("Error opening lock file %s: %s"),
 			lockfilename, strerror(errno));
-	    return -1;
+	    goto free_the_name;
 	}
+
+	lockbuf = charalloc(8192);
 	do {
 	    readamt = read(lockfd, &lockbuf[readtot], BUFSIZ);
 	    readtot += readamt;
@@ -287,37 +348,54 @@ int do_lockfile(const char *filename)
 	if (readtot < 48) {
 	    statusbar(_("Error reading lock file %s: Not enough data read"),
 			lockfilename);
-	    return -1;
+	    free(lockbuf);
+	    goto free_the_name;
 	}
+
 	strncpy(lockprog, &lockbuf[2], 10);
 	lockpid = (unsigned char)lockbuf[25] * 256 + (unsigned char)lockbuf[24];
 	strncpy(lockuser, &lockbuf[28], 16);
+	free(lockbuf);
+
 #ifdef DEBUG
 	fprintf(stderr, "lockpid = %d\n", lockpid);
 	fprintf(stderr, "program name which created this lock file should be %s\n", lockprog);
 	fprintf(stderr, "user which created this lock file should be %s\n", lockuser);
 #endif
+	promptstr = charalloc(128);
 	/* TRANSLATORS: The second %s is the name of the user, the third that of the editor. */
 	sprintf(promptstr, _("File %s is being edited (by %s with %s, PID %d); continue?"),
 			filename, lockuser, lockprog, lockpid);
 	ans = do_yesno_prompt(FALSE, promptstr);
+	free(promptstr);
+
 	if (ans < 1) {
 	    blank_statusbar();
-	    return -1;
+	    goto free_the_name;
 	}
-    } else {
-	lockfiledir = mallocstrcpy(NULL, lockfilename);
-	lockfiledir = dirname(lockfiledir);
-	if (stat(lockfiledir, &fileinfo) == -1) {
-	    statusbar(_("Error writing lock file: Directory \'%s\' doesn't exist"),
-		lockfiledir);
-	    free(lockfiledir);
-	    return 0;
-	}
-	free(lockfiledir);
     }
 
-    return write_lockfile(lockfilename, filename, FALSE);
+    retval = write_lockfile(lockfilename, filename, FALSE);
+
+  free_the_name:
+    if (retval < 1)
+	free(lockfilename);
+
+    return retval;
+}
+
+/* Perform a stat call on the given filename, allocating a stat struct
+ * if necessary.  On success, *pstat points to the stat's result.  On
+ * failure, *pstat is freed and made NULL. */
+void stat_with_alloc(const char *filename, struct stat **pstat)
+{
+    if (*pstat == NULL)
+	*pstat = (struct stat *)nmalloc(sizeof(struct stat));
+
+    if (stat(filename, *pstat) != 0) {
+	free(*pstat);
+	*pstat = NULL;
+    }
 }
 #endif /* !NANO_TINY */
 
@@ -363,25 +441,28 @@ bool open_buffer(const char *filename, bool undoable)
     }
 
     /* If we're going to load into a new buffer, first create the new
-     * buffer and lock the corresponding file. */
+     * buffer and (if possible) lock the corresponding file. */
     if (new_buffer) {
 	make_new_buffer();
 
+	if (!has_valid_path(filename))
+	    quiet = TRUE;
 #ifndef NANO_TINY
-	if (ISSET(LOCKING) && filename[0] != '\0') {
-	    int lockstatus = do_lockfile(filename);
-	    if (lockstatus < 0) {
+	else {
+	    if (ISSET(LOCKING) && filename[0] != '\0') {
+		int lockstatus = do_lockfile(filename);
+		if (lockstatus < 0) {
 #ifndef DISABLE_MULTIBUFFER
-		if (openfile->next) {
-		    close_buffer(TRUE);
-		    return FALSE;
-		}
+		    if (openfile->next) {
+			close_buffer(TRUE);
+			return FALSE;
+		    }
 #endif
-	    } else if (lockstatus == 0) {
-		quiet = TRUE;
+		} else if (lockstatus == 0)
+		    quiet = TRUE;
 	    }
 	}
-#endif
+#endif /* !NANO_TINY */
     }
 
     /* If the filename isn't blank, and we are not in NOREAD_MODE,
@@ -399,11 +480,8 @@ bool open_buffer(const char *filename, bool undoable)
     if (rc > 0) {
 	read_file(f, rc, filename, undoable, new_buffer);
 #ifndef NANO_TINY
-	if (openfile->current_stat == NULL) {
-	    openfile->current_stat =
-		(struct stat *)nmalloc(sizeof(struct stat));
-	    stat(filename, openfile->current_stat);
-	}
+	if (openfile->current_stat == NULL)
+	    stat_with_alloc(filename, &openfile->current_stat);
 #endif
     }
 
@@ -533,7 +611,9 @@ bool close_buffer(bool quiet)
 	return FALSE;
 
 #ifndef DISABLE_HISTORIES
-    update_poshistory(openfile->filename, openfile->current->lineno, xplustabs() + 1);
+    if (ISSET(POS_HISTORY))
+	update_poshistory(openfile->filename,
+			openfile->current->lineno, xplustabs() + 1);
 #endif
 
     /* Switch to the next file buffer. */
@@ -593,42 +673,42 @@ int is_file_writable(const char *filename)
  * Then attach this line after prevnode. */
 filestruct *read_line(char *buf, size_t buf_len, filestruct *prevnode)
 {
-    filestruct *fileptr = (filestruct *)nmalloc(sizeof(filestruct));
+    filestruct *freshline = (filestruct *)nmalloc(sizeof(filestruct));
 
     /* Convert nulls to newlines.  buf_len is the string's real length. */
     unsunder(buf, buf_len);
 
     assert(openfile->fileage != NULL && strlen(buf) == buf_len);
 
-    fileptr->data = mallocstrcpy(NULL, buf);
+    freshline->data = mallocstrcpy(NULL, buf);
 
 #ifndef NANO_TINY
     /* If it's a DOS file ("\r\n"), and file conversion isn't disabled,
-     * strip the '\r' part from fileptr->data. */
+     * strip the '\r' part from the data. */
     if (!ISSET(NO_CONVERT) && buf_len > 0 && buf[buf_len - 1] == '\r')
-	fileptr->data[buf_len - 1] = '\0';
+	freshline->data[buf_len - 1] = '\0';
 #endif
 
 #ifndef DISABLE_COLOR
-    fileptr->multidata = NULL;
+    freshline->multidata = NULL;
 #endif
 
-    fileptr->prev = prevnode;
+    freshline->prev = prevnode;
 
     if (prevnode == NULL) {
 	/* Special case: we're inserting into the first line. */
-	fileptr->next = openfile->fileage;
-	openfile->fileage = fileptr;
-	fileptr->lineno = 1;
+	freshline->next = openfile->fileage;
+	openfile->fileage = freshline;
+	freshline->lineno = 1;
 	/* Make sure that our edit window stays on the first line. */
-	openfile->edittop = fileptr;
+	openfile->edittop = freshline;
     } else {
-	prevnode->next = fileptr;
-	fileptr->next = NULL;
-	fileptr->lineno = prevnode->lineno + 1;
+	prevnode->next = freshline;
+	freshline->next = NULL;
+	freshline->lineno = prevnode->lineno + 1;
     }
 
-    return fileptr;
+    return freshline;
 }
 
 /* Read an open file into the current buffer.  f should be set to the
@@ -922,16 +1002,18 @@ int open_file(const char *filename, bool newfie, bool quiet, FILE **f)
      * permissions, just try the relative one. */
     if (full_filename == NULL || (stat(full_filename, &fileinfo) == -1 &&
 		stat(filename, &fileinfo2) != -1))
-	full_filename = mallocstrcpy(NULL, filename);
+	full_filename = mallocstrcpy(full_filename, filename);
 
     if (stat(full_filename, &fileinfo) == -1) {
+	/* All cases below return. */
+	free(full_filename);
+
 	/* Well, maybe we can open the file even if the OS says it's
 	 * not there. */
 	if ((fd = open(filename, O_RDONLY)) != -1) {
 	    if (!quiet)
 		statusbar(_("Reading File"));
-	    free(full_filename);
-	    return 0;
+	    return fd;
 	}
 
 	if (newfie) {
@@ -944,6 +1026,8 @@ int open_file(const char *filename, bool newfie, bool quiet, FILE **f)
 	return -1;
     } else if (S_ISDIR(fileinfo.st_mode) || S_ISCHR(fileinfo.st_mode) ||
 		S_ISBLK(fileinfo.st_mode)) {
+	free(full_filename);
+
 	/* Don't open directories, character files, or block files.
 	 * Sorry, /dev/sndstat! */
 	statusbar(S_ISDIR(fileinfo.st_mode) ?
@@ -952,6 +1036,7 @@ int open_file(const char *filename, bool newfie, bool quiet, FILE **f)
 	beep();
 	return -1;
     } else if ((fd = open(full_filename, O_RDONLY)) == -1) {
+	free(full_filename);
 	statusbar(_("Error reading %s: %s"), filename, strerror(errno));
 	beep();
 	return -1;
@@ -1196,20 +1281,23 @@ void do_insertfile(
 		open_buffer(answer, TRUE);
 	    }
 
-#if !defined(DISABLE_MULTIBUFFER) && !defined(DISABLE_HISTORIES)
+#ifndef DISABLE_MULTIBUFFER
 	    if (ISSET(MULTIBUFFER)) {
 		/* Update the screen to account for the current buffer. */
 		display_buffer();
 
-		ssize_t savedposline, savedposcol;
-		if (ISSET(POS_HISTORY) &&
+#ifndef DISABLE_HISTORIES
+		if (ISSET(POS_HISTORY)) {
+		    ssize_t priorline, priorcol;
 #ifndef NANO_TINY
-			!execute &&
+		    if (!execute)
 #endif
-			check_poshistory(answer, &savedposline, &savedposcol))
-		    do_gotolinecolumn(savedposline, savedposcol, FALSE, FALSE);
+		    if (check_poshistory(answer, &priorline, &priorcol))
+			do_gotolinecolumn(priorline, priorcol, FALSE, FALSE);
+		}
+#endif /* !DISABLE_HISTORIES */
 	    } else
-#endif /* !DISABLE_MULTIBUFFER && !DISABLE_HISTORIES */
+#endif /* !DISABLE_MULTIBUFFER */
 	    {
 		filestruct *top_save = openfile->fileage;
 
@@ -1416,10 +1504,10 @@ char *get_full_path(const char *origpath)
 	     * where we were before.  We don't check for a chdir()
 	     * error, since we can do nothing if we get one. */
 	    IGNORE_CALL_RESULT(chdir(d_here));
-
-	    /* Free d_here, since we're done using it. */
-	    free(d_here);
 	}
+
+	/* Free d_here, since we're done using it. */
+	free(d_here);
     }
 
     /* At this point, if path_only is FALSE and d_there isn't NULL,
@@ -1446,7 +1534,6 @@ char *check_writable_directory(const char *path)
 {
     char *full_path = get_full_path(path);
 
-    /* If get_full_path() fails, return NULL. */
     if (full_path == NULL)
 	return NULL;
 
@@ -1457,7 +1544,6 @@ char *check_writable_directory(const char *path)
 	return NULL;
     }
 
-    /* Otherwise, return the full path. */
     return full_path;
 }
 
@@ -1685,17 +1771,12 @@ bool write_file(const char *name, FILE *f_open, bool tmp, append_type
 	/* The result of stat().  TRUE if the file exists, FALSE
 	 * otherwise.  If name is a link that points nowhere, realexists
 	 * is FALSE. */
+#endif
     struct stat st;
 	/* The status fields filled in by stat(). */
-#endif
-    bool anyexists;
-	/* The result of lstat().  The same as realexists, unless name
-	 * is a link. */
-    struct stat lst;
-	/* The status fields filled in by lstat(). */
     char *realname;
 	/* name after tilde expansion. */
-    FILE *f = NULL;
+    FILE *f = f_open;
 	/* The actual file, realname, we are writing to. */
     char *tempname = NULL;
 	/* The name of the temporary file we write to on prepend. */
@@ -1705,9 +1786,6 @@ bool write_file(const char *name, FILE *f_open, bool tmp, append_type
     if (*name == '\0')
 	return -1;
 
-    if (f_open != NULL)
-	f = f_open;
-
     if (!tmp)
 	titlebar(NULL);
 
@@ -1722,20 +1800,10 @@ bool write_file(const char *name, FILE *f_open, bool tmp, append_type
     }
 #endif
 
-    anyexists = (lstat(realname, &lst) != -1);
-
     /* If the temp file exists and isn't already open, give up. */
-    if (tmp && anyexists && f_open == NULL)
+    if (tmp && (lstat(realname, &st) != -1) && f_open == NULL)
 	goto cleanup_and_exit;
 
-    /* If NOFOLLOW_SYMLINKS is set, it doesn't make sense to prepend or
-     * append to a symlink.  Here we warn about the contradiction. */
-    if (ISSET(NOFOLLOW_SYMLINKS) && anyexists && S_ISLNK(lst.st_mode)) {
-	statusbar(
-		_("Cannot prepend or append to a symlink with --nofollow set"));
-	goto cleanup_and_exit;
-    }
-
 #ifndef NANO_TINY
     /* Check whether the file (at the end of the symlink) exists. */
     realexists = (stat(realname, &st) != -1);
@@ -1744,10 +1812,8 @@ bool write_file(const char *name, FILE *f_open, bool tmp, append_type
      * specified it interactively), stat and save the value now,
      * or else we will chase null pointers when we do modtime checks,
      * preserve file times, and so on, during backup. */
-    if (openfile->current_stat == NULL && !tmp && realexists) {
-	openfile->current_stat = (struct stat *)nmalloc(sizeof(struct stat));
-	stat(realname, openfile->current_stat);
-    }
+    if (openfile->current_stat == NULL && !tmp && realexists)
+	stat_with_alloc(realname, &openfile->current_stat);
 
     /* We backup only if the backup toggle is set, the file isn't
      * temporary, and the file already exists.  Furthermore, if we
@@ -1925,15 +1991,6 @@ bool write_file(const char *name, FILE *f_open, bool tmp, append_type
     skip_backup:
 #endif /* !NANO_TINY */
 
-    /* If NOFOLLOW_SYMLINKS is set and the file is a link, we aren't
-     * doing prepend or append.  So we delete the link first, and just
-     * overwrite. */
-    if (ISSET(NOFOLLOW_SYMLINKS) && anyexists && S_ISLNK(lst.st_mode) &&
-		unlink(realname) == -1) {
-	statusbar(_("Error writing %s: %s"), realname, strerror(errno));
-	goto cleanup_and_exit;
-    }
-
     if (f_open == NULL) {
 	original_umask = umask(0);
 
@@ -2133,12 +2190,9 @@ bool write_file(const char *name, FILE *f_open, bool tmp, append_type
 	}
 
 #ifndef NANO_TINY
-	/* Update current_stat to reference the file as it is now. */
-	if (openfile->current_stat == NULL)
-	    openfile->current_stat =
-		(struct stat *)nmalloc(sizeof(struct stat));
 	if (!openfile->mark_set)
-	    stat(realname, openfile->current_stat);
+	    /* Get or update the stat info to reflect the current state. */
+	    stat_with_alloc(realname, &openfile->current_stat);
 #endif
 
 	statusbar(P_("Wrote %lu line", "Wrote %lu lines",
@@ -2259,9 +2313,8 @@ int do_writeout(bool exiting)
 		  (append == APPEND) ? _("File Name to Append to") :
 		  _("File Name to Write");
 
-	/* If we're using restricted mode, the filename isn't blank,
-	 * and we're at the "Write File" prompt, disable tab
-	 * completion. */
+	/* If we're using restricted mode, and the filename isn't blank,
+	 * disable tab completion. */
 	i = do_prompt(!ISSET(RESTRICTED) ||
 		openfile->filename[0] == '\0',
 #ifndef DISABLE_TABCOMP
@@ -3107,69 +3160,93 @@ void save_history(void)
 /* Save the recorded last file positions to ~/.nano/filepos_history. */
 void save_poshistory(void)
 {
-    char *poshist;
+    char *poshist = poshistfilename();
     char *statusstr = NULL;
     poshiststruct *posptr;
+    FILE *hist;
 
-    poshist = poshistfilename();
+    if (poshist == NULL)
+	return;
 
-    if (poshist != NULL) {
-	FILE *hist = fopen(poshist, "wb");
+    hist = fopen(poshist, "wb");
 
-	if (hist == NULL)
-	    fprintf(stderr, _("Error writing %s: %s\n"), poshist,
-			strerror(errno));
-	else {
-	    /* Make sure no one else can read from or write to the
-	     * history file. */
-	    chmod(poshist, S_IRUSR | S_IWUSR);
+    if (hist == NULL)
+	fprintf(stderr, _("Error writing %s: %s\n"), poshist, strerror(errno));
+    else {
+	/* Don't allow others to read or write the history file. */
+	chmod(poshist, S_IRUSR | S_IWUSR);
 
-	    for (posptr = poshistory; posptr != NULL; posptr = posptr->next) {
-		statusstr = charalloc(strlen(posptr->filename) + 2 * sizeof(ssize_t) + 4);
-		sprintf(statusstr, "%s %ld %ld\n", posptr->filename, (long)posptr->lineno,
+	for (posptr = position_history; posptr != NULL; posptr = posptr->next) {
+	    statusstr = charalloc(strlen(posptr->filename) + 2 * sizeof(ssize_t) + 4);
+	    sprintf(statusstr, "%s %ld %ld\n", posptr->filename, (long)posptr->lineno,
 			(long)posptr->xno);
-		if (fwrite(statusstr, sizeof(char), strlen(statusstr), hist) < strlen(statusstr))
-		    fprintf(stderr, _("Error writing %s: %s\n"), poshist,
-				strerror(errno));
-		free(statusstr);
-	    }
-	    fclose(hist);
+	    if (fwrite(statusstr, sizeof(char), strlen(statusstr), hist) < strlen(statusstr))
+		fprintf(stderr, _("Error writing %s: %s\n"), poshist, strerror(errno));
+	    free(statusstr);
 	}
-	free(poshist);
+	fclose(hist);
     }
+    free(poshist);
 }
 
 /* Update the recorded last file positions, given a filename, a line
  * and a column.  If no entry is found, add a new one at the end. */
 void update_poshistory(char *filename, ssize_t lineno, ssize_t xpos)
 {
-    poshiststruct *posptr, *posprev = NULL;
+    poshiststruct *posptr, *theone, *posprev = NULL;
     char *fullpath = get_full_path(filename);
 
-    if (fullpath == NULL)
+    if (fullpath == NULL || fullpath[strlen(fullpath) - 1] == '/') {
+	free(fullpath);
 	return;
+    }
 
-    for (posptr = poshistory; posptr != NULL; posptr = posptr->next) {
-	if (!strcmp(posptr->filename, fullpath)) {
-	    posptr->lineno = lineno;
-	    posptr->xno = xpos;
-	    free(fullpath);
-	    return;
-	}
+    /* Look for a matching filename in the list. */
+    for (posptr = position_history; posptr != NULL; posptr = posptr->next) {
+	if (!strcmp(posptr->filename, fullpath))
+	    break;
 	posprev = posptr;
     }
 
-    /* Didn't find it, make a new node yo! */
-    posptr = (poshiststruct *)nmalloc(sizeof(poshiststruct));
-    posptr->filename = mallocstrcpy(NULL, fullpath);
-    posptr->lineno = lineno;
-    posptr->xno = xpos;
-    posptr->next = NULL;
+    /* Don't record files that have the default cursor position. */
+    if (lineno == 1 && xpos == 1) {
+	if (posptr != NULL) {
+	    if (posprev == NULL)
+		position_history = posptr->next;
+	    else
+		posprev->next = posptr->next;
+	    free(posptr->filename);
+	    free(posptr);
+	}
+	free(fullpath);
+	return;
+    }
 
-    if (!poshistory)
-	poshistory = posptr;
-    else
-	posprev->next = posptr;
+    theone = posptr;
+
+    /* If we didn't find it, make a new node; otherwise, if we're
+     * not at the end, move the matching one to the end. */
+    if (theone == NULL) {
+	theone = (poshiststruct *)nmalloc(sizeof(poshiststruct));
+	theone->filename = mallocstrcpy(NULL, fullpath);
+	if (position_history == NULL)
+	    position_history = theone;
+	else
+	    posprev->next = theone;
+    } else if (posptr->next != NULL) {
+	if (posprev == NULL)
+	    position_history = posptr->next;
+	else
+	    posprev->next = posptr->next;
+	while (posptr->next != NULL)
+	    posptr = posptr->next;
+	posptr->next = theone;
+    }
+
+    /* Store the last cursor position. */
+    theone->lineno = lineno;
+    theone->xno = xpos;
+    theone->next = NULL;
 
     free(fullpath);
 }
@@ -3185,7 +3262,7 @@ int check_poshistory(const char *file, ssize_t *line, ssize_t *column)
     if (fullpath == NULL)
 	return 0;
 
-    for (posptr = poshistory; posptr != NULL; posptr = posptr->next) {
+    for (posptr = position_history; posptr != NULL; posptr = posptr->next) {
 	if (!strcmp(posptr->filename, fullpath)) {
 	    *line = posptr->lineno;
 	    *column = posptr->xno;
@@ -3200,59 +3277,64 @@ int check_poshistory(const char *file, ssize_t *line, ssize_t *column)
 /* Load the recorded file positions from ~/.nano/filepos_history. */
 void load_poshistory(void)
 {
-    char *nanohist = poshistfilename();
+    char *poshist = poshistfilename();
+    FILE *hist;
 
-    /* Assume do_rcfile() has reported a missing home directory. */
-    if (nanohist != NULL) {
-	FILE *hist = fopen(nanohist, "rb");
+    /* If the home directory is missing, do_rcfile() will have reported it. */
+    if (poshist == NULL)
+	return;
 
-	if (hist == NULL) {
-	    if (errno != ENOENT) {
-		/* Don't save history when we quit. */
-		UNSET(POS_HISTORY);
-		history_error(N_("Error reading %s: %s"), nanohist,
-			strerror(errno));
-	    }
-	} else {
-	    char *line = NULL, *lineptr, *xptr;
-	    size_t buf_len = 0;
-	    ssize_t read, lineno, xno;
-	    poshiststruct *posptr;
+    hist = fopen(poshist, "rb");
 
-	    /* Read and parse each line, and put the data into the
-	     * positions history structure. */
-	    while ((read = getline(&line, &buf_len, hist)) >= 0) {
-		if (read > 0 && line[read - 1] == '\n') {
-		    read--;
-		    line[read] = '\0';
-		}
-		if (read > 0)
-		    unsunder(line, read);
-		lineptr = parse_next_word(line);
-		xptr = parse_next_word(lineptr);
-		lineno = atoi(lineptr);
-		xno = atoi(xptr);
-		if (poshistory == NULL) {
-		    poshistory = (poshiststruct *)nmalloc(sizeof(poshiststruct));
-		    poshistory->filename = mallocstrcpy(NULL, line);
-		    poshistory->lineno = lineno;
-		    poshistory->xno = xno;
-		    poshistory->next = NULL;
-		} else {
-		    for (posptr = poshistory; posptr->next != NULL; posptr = posptr->next)
-			;
-		    posptr->next = (poshiststruct *)nmalloc(sizeof(poshiststruct));
-		    posptr->next->filename = mallocstrcpy(NULL, line);
-		    posptr->next->lineno = lineno;
-		    posptr->next->xno = xno;
-		    posptr->next->next = NULL;
-		}
-	    }
-
-	    fclose(hist);
-	    free(line);
+    if (hist == NULL) {
+	if (errno != ENOENT) {
+	    /* When reading failed, don't save history when we quit. */
+	    UNSET(POS_HISTORY);
+	    history_error(N_("Error reading %s: %s"), poshist, strerror(errno));
 	}
-	free(nanohist);
+    } else {
+	char *line = NULL, *lineptr, *xptr;
+	size_t buf_len = 0;
+	ssize_t read, count = 0;
+	poshiststruct *record_ptr = NULL, *newrecord;
+
+	/* Read and parse each line, and store the extracted data. */
+	while ((read = getline(&line, &buf_len, hist)) > 2) {
+	    if (line[read - 1] == '\n')
+		line[--read] = '\0';
+	    unsunder(line, read);
+
+	    lineptr = parse_next_word(line);
+	    xptr = parse_next_word(lineptr);
+
+	    /* Create a new position record. */
+	    newrecord = (poshiststruct *)nmalloc(sizeof(poshiststruct));
+	    newrecord->filename = mallocstrcpy(NULL, line);
+	    newrecord->lineno = atoi(lineptr);
+	    newrecord->xno = atoi(xptr);
+	    newrecord->next = NULL;
+
+	    /* Add the record to the list. */
+	    if (position_history == NULL)
+		position_history = newrecord;
+	    else
+		record_ptr->next = newrecord;
+
+	    record_ptr = newrecord;
+
+	    /* Impose a limit, so the file will not grow indefinitely. */
+	    if (++count > 200) {
+		poshiststruct *drop_record = position_history;
+
+		position_history = position_history->next;
+
+		free(drop_record->filename);
+		free(drop_record);
+	    }
+	}
+	fclose(hist);
+	free(line);
     }
+    free(poshist);
 }
 #endif /* !DISABLE_HISTORIES */
