@@ -1,4 +1,4 @@
-/* $Id: global.c 5537 2016-01-07 15:42:44Z bens $ */
+/* $Id: global.c 5607 2016-02-05 12:27:54Z bens $ */
 /**************************************************************************
  *   global.c                                                             *
  *                                                                        *
@@ -190,7 +190,7 @@ filestruct *replaceage = NULL;
 	/* The top of the replace string history list. */
 filestruct *replacebot = NULL;
 	/* The bottom of the replace string history list. */
-poshiststruct *poshistory;
+poshiststruct *position_history = NULL;
 	/* The cursor position history list. */
 #endif
 
@@ -480,7 +480,7 @@ void print_sclist(void)
 const char *exit_tag = N_("Exit");
 const char *close_tag = N_("Close");
 const char *uncut_tag = N_("Uncut Text");
-#ifndef DISABLE_JUSITIFY
+#ifndef DISABLE_JUSTIFY
 const char *unjust_tag = N_("Unjustify");
 #endif
 
@@ -552,11 +552,11 @@ void shortcut_init(void)
     const char *nano_undo_msg = N_("Undo the last operation");
     const char *nano_redo_msg = N_("Redo the last undone operation");
 #endif
-    const char *nano_forward_msg = N_("Go forward one character");
     const char *nano_back_msg = N_("Go back one character");
+    const char *nano_forward_msg = N_("Go forward one character");
 #ifndef NANO_TINY
-    const char *nano_nextword_msg = N_("Go forward one word");
     const char *nano_prevword_msg = N_("Go back one word");
+    const char *nano_nextword_msg = N_("Go forward one word");
 #endif
     const char *nano_prevline_msg = N_("Go to previous line");
     const char *nano_nextline_msg = N_("Go to next line");
@@ -908,6 +908,7 @@ void shortcut_init(void)
 
 #ifndef NANO_TINY
     add_to_funcs(do_cut_prev_word, MMAIN,
+	/* TRANSLATORS: The next two strings refer to cutting words. */
 	N_("Cut Left"), IFSCHELP(nano_cut_word_left_msg), TOGETHER, NOVIEW);
     add_to_funcs(do_cut_next_word, MMAIN,
 	N_("Cut Right"), IFSCHELP(nano_cut_word_right_msg), TOGETHER, NOVIEW);
@@ -1014,8 +1015,8 @@ void shortcut_init(void)
 #endif
 
 #ifndef DISABLE_COLOR
-    /* TRANSLATORS: Try to keep the next two strings at most 20 characters. */
     add_to_funcs(do_page_up, MLINTER,
+	/* TRANSLATORS: Try to keep the next two strings at most 20 characters. */
 	N_("Prev Lint Msg"), IFSCHELP(nano_prevlint_msg), TOGETHER, VIEW);
     add_to_funcs(do_page_down, MLINTER,
 	N_("Next Lint Msg"), IFSCHELP(nano_nextlint_msg), TOGETHER, VIEW);
@@ -1260,7 +1261,7 @@ void set_spell_shortcuts(void)
 	replace_scs_for(do_linter, do_spell);
 #endif
 }
-#endif
+#endif /* !DISABLE_COLOR */
 
 const subnfunc *sctofunc(const sc *s)
 {
