@@ -1,4 +1,4 @@
-/* $Id: nano.c 5542 2016-01-10 16:18:43Z bens $ */
+/* $Id: nano.c 5628 2016-02-10 20:16:50Z bens $ */
 /**************************************************************************
  *   nano.c                                                               *
  *                                                                        *
@@ -561,6 +561,8 @@ void delete_opennode(openfilestruct *fileptr)
 #ifndef NANO_TINY
     free(fileptr->current_stat);
     free(fileptr->lock_filename);
+    /* Free the undo stack. */
+    discard_until(NULL, fileptr);
 #endif
     free(fileptr);
 }
@@ -918,7 +920,6 @@ void usage(void)
     print_opt("-i", "--autoindent", N_("Automatically indent new lines"));
     print_opt("-k", "--cut", N_("Cut from cursor to end of line"));
 #endif
-    print_opt("-l", "--nofollow", N_("Don't follow symbolic links, overwrite"));
 #ifndef DISABLE_MOUSE
     print_opt("-m", "--mouse", N_("Enable the use of the mouse"));
 #endif
@@ -1105,7 +1106,6 @@ void no_current_file_name_warning(void)
     beep();
 
     /* Ensure that we see the warning. */
-    doupdate();
     napms(1800);
 
     curs_set(1);
@@ -1346,9 +1346,6 @@ RETSIGTYPE do_continue(int signal)
     /* Restore the terminal to its previous state. */
     terminal_init();
 
-    /* Turn the cursor back on for sure. */
-    curs_set(1);
-
     /* Redraw the contents of the windows that need it. */
     blank_statusbar();
     wnoutrefresh(bottomwin);
@@ -1407,9 +1404,6 @@ void regenerate_screen(void)
     /* Restore the terminal to its previous state. */
     terminal_init();
 
-    /* Turn the cursor back on for sure. */
-    curs_set(1);
-
     /* Do the equivalent of what both mutt and Minimum Profit do:
      * Reinitialize all the windows based on the new screen
      * dimensions. */
@@ -2008,7 +2002,6 @@ int main(int argc, char **argv)
 	{"constantshow", 0, NULL, 'c'},
 	{"rebinddelete", 0, NULL, 'd'},
 	{"help", 0, NULL, 'h'},
-	{"nofollow", 0, NULL, 'l'},
 #ifndef DISABLE_MOUSE
 	{"mouse", 0, NULL, 'm'},
 #endif
@@ -2212,9 +2205,6 @@ int main(int argc, char **argv)
 		SET(CUT_TO_END);
 		break;
 #endif
-	    case 'l':
-		SET(NOFOLLOW_SYMLINKS);
-		break;
 #ifndef DISABLE_MOUSE
 	    case 'm':
 		SET(USE_MOUSE);
@@ -2538,9 +2528,6 @@ int main(int argc, char **argv)
     /* Set up the terminal state. */
     terminal_init();
 
-    /* Turn the cursor on for sure. */
-    curs_set(1);
-
 #ifdef DEBUG
     fprintf(stderr, "Main: set up windows\n");
 #endif
@@ -2626,7 +2613,7 @@ int main(int argc, char **argv)
 		    icol = 0;
 		}
 #ifndef DISABLE_HISTORIES
-		else {
+		else if (ISSET(POS_HISTORY)) {
 		    ssize_t savedposline, savedposcol;
 		    /* If edited before, restore the last cursor position. */
 		    if (check_poshistory(argv[i], &savedposline, &savedposcol))
@@ -2664,7 +2651,7 @@ int main(int argc, char **argv)
     if (startline > 0 || startcol > 0)
 	do_gotolinecolumn(startline, startcol, FALSE, FALSE);
 #ifndef DISABLE_HISTORIES
-    else {
+    else if (ISSET(POS_HISTORY)) {
 	ssize_t savedposline, savedposcol;
 	/* If the file was edited before, restore the last cursor position. */
 	if (check_poshistory(argv[optind], &savedposline, &savedposcol))
@@ -2681,10 +2668,6 @@ int main(int argc, char **argv)
 	reset_cursor();
 	wnoutrefresh(edit);
 
-	/* Just in case we were at the statusbar prompt, make sure the
-	 * statusbar cursor position is reset. */
-	do_prompt_abort();
-
 	/* If constant cursor position display is on, and there are no
 	 * keys waiting in the input buffer, display the current cursor
 	 * position on the statusbar. */
