@@ -1,4 +1,4 @@
-/* $Id: prompt.c 5530 2016-01-04 09:52:43Z bens $ */
+/* $Id: prompt.c 5620 2016-02-07 13:41:46Z bens $ */
 /**************************************************************************
  *   prompt.c                                                             *
  *                                                                        *
@@ -37,9 +37,6 @@ static size_t old_statusbar_x = (size_t)-1;
 	/* The old cursor position in answer, if any. */
 static size_t old_pww = (size_t)-1;
 	/* The old place we want in answer, if any. */
-static bool reset_statusbar_x = FALSE;
-	/* Should we reset the cursor position at the statusbar
-	 * prompt? */
 
 /* Read in a character, interpret it as a shortcut or toggle if
  * necessary, and return it.
@@ -147,14 +144,15 @@ int do_statusbar_input(bool *ran_func, bool *finished,
 	if (have_shortcut) {
 	    if (s->scfunc == do_tab || s->scfunc == do_enter)
 		;
-	    else if (s->scfunc == total_refresh)
-		total_statusbar_refresh(refresh_func);
-	    else if (s->scfunc == do_cut_text_void) {
+	    else if (s->scfunc == total_refresh) {
+		total_redraw();
+		refresh_func();
+	    } else if (s->scfunc == do_cut_text_void) {
 		/* If we're using restricted mode, the filename
 		 * isn't blank, and we're at the "Write File"
 		 * prompt, disable Cut. */
-		if (!ISSET(RESTRICTED) || openfile->filename[0] ==
-			'\0' || currmenu != MWRITEFILE)
+		if (!ISSET(RESTRICTED) || openfile->filename[0] == '\0' ||
+			currmenu != MWRITEFILE)
 		    do_statusbar_cut_text();
 	    } else if (s->scfunc == do_left)
 		do_statusbar_left();
@@ -162,9 +160,9 @@ int do_statusbar_input(bool *ran_func, bool *finished,
 		do_statusbar_right();
 #ifndef NANO_TINY
 	    else if (s->scfunc == do_prev_word_void)
-		do_statusbar_prev_word(FALSE);
+		do_statusbar_prev_word();
 	    else if (s->scfunc == do_next_word_void)
-		do_statusbar_next_word(FALSE);
+		do_statusbar_next_word();
 #endif
 	    else if (s->scfunc == do_home)
 		do_statusbar_home();
@@ -194,15 +192,15 @@ int do_statusbar_input(bool *ran_func, bool *finished,
 		/* If we're using restricted mode, the filename
 		 * isn't blank, and we're at the "Write File"
 		 * prompt, disable Delete. */
-		if (!ISSET(RESTRICTED) || openfile->filename[0] ==
-			'\0' || currmenu != MWRITEFILE)
+		if (!ISSET(RESTRICTED) || openfile->filename[0] == '\0' ||
+			currmenu != MWRITEFILE)
 		    do_statusbar_delete();
 	    } else if (s->scfunc == do_backspace) {
 		/* If we're using restricted mode, the filename
 		 * isn't blank, and we're at the "Write File"
 		 * prompt, disable Backspace. */
-		if (!ISSET(RESTRICTED) || openfile->filename[0] ==
-			'\0' || currmenu != MWRITEFILE)
+		if (!ISSET(RESTRICTED) || openfile->filename[0] == '\0' ||
+			currmenu != MWRITEFILE)
 		    do_statusbar_backspace();
 	    } else {
 		/* Handle any other shortcut in the current menu, setting
@@ -233,8 +231,7 @@ int do_statusbar_mouse(void)
     int retval = get_mouseinput(&mouse_x, &mouse_y, TRUE);
 
     /* We can click on the statusbar window text to move the cursor. */
-    if (retval == 0 && wmouse_trafo(bottomwin, &mouse_y, &mouse_x,
-	FALSE)) {
+    if (retval == 0 && wmouse_trafo(bottomwin, &mouse_y, &mouse_x, FALSE)) {
 	size_t start_col;
 
 	assert(prompt != NULL);
@@ -243,15 +240,10 @@ int do_statusbar_mouse(void)
 
 	/* Move to where the click occurred. */
 	if (mouse_x >= start_col && mouse_y == 0) {
-	    size_t pww_save = statusbar_pww;
-
 	    statusbar_x = actual_x(answer,
 			get_statusbar_page_start(start_col, start_col +
 			statusbar_xplustabs()) + mouse_x - start_col);
-	    statusbar_pww = statusbar_xplustabs();
-
-	    if (need_statusbar_update(pww_save))
-		update_statusbar_line(answer, statusbar_x);
+	    update_bar_if_needed();
 	}
     }
 
@@ -310,9 +302,8 @@ void do_statusbar_output(char *output, size_t output_len, bool
 
 	assert(statusbar_x <= answer_len);
 
-	charmove(answer + statusbar_x + char_buf_len,
-		answer + statusbar_x, answer_len - statusbar_x +
-		char_buf_len);
+	charmove(answer + statusbar_x + char_buf_len, answer + statusbar_x,
+			answer_len - statusbar_x + char_buf_len);
 	strncpy(answer + statusbar_x, char_buf, char_buf_len);
 	answer_len += char_buf_len;
 
@@ -323,62 +314,29 @@ void do_statusbar_output(char *output, size_t output_len, bool
 
     statusbar_pww = statusbar_xplustabs();
 
-    update_statusbar_line(answer, statusbar_x);
+    update_the_statusbar();
 }
 
-/* Move to the beginning of the prompt text.  If the SMART_HOME flag is
- * set, move to the first non-whitespace character of the prompt text if
- * we're not already there, or to the beginning of the prompt text if we
- * are. */
+/* Move to the beginning of the prompt text. */
 void do_statusbar_home(void)
 {
-    size_t pww_save = statusbar_pww;
-
-#ifndef NANO_TINY
-    if (ISSET(SMART_HOME)) {
-	size_t statusbar_x_save = statusbar_x;
-
-	statusbar_x = indent_length(answer);
-
-	if (statusbar_x == statusbar_x_save ||
-		statusbar_x == strlen(answer))
-	    statusbar_x = 0;
-
-	statusbar_pww = statusbar_xplustabs();
-    } else
-#endif
-    {
-	statusbar_x = 0;
-	statusbar_pww = statusbar_xplustabs();
-    }
-
-    if (need_statusbar_update(pww_save))
-	update_statusbar_line(answer, statusbar_x);
+    statusbar_x = 0;
+    update_bar_if_needed();
 }
 
 /* Move to the end of the prompt text. */
 void do_statusbar_end(void)
 {
-    size_t pww_save = statusbar_pww;
-
     statusbar_x = strlen(answer);
-    statusbar_pww = statusbar_xplustabs();
-
-    if (need_statusbar_update(pww_save))
-	update_statusbar_line(answer, statusbar_x);
+    update_bar_if_needed();
 }
 
 /* Move left one character. */
 void do_statusbar_left(void)
 {
     if (statusbar_x > 0) {
-	size_t pww_save = statusbar_pww;
-
 	statusbar_x = move_mbleft(answer, statusbar_x);
-	statusbar_pww = statusbar_xplustabs();
-
-	if (need_statusbar_update(pww_save))
-	    update_statusbar_line(answer, statusbar_x);
+	update_bar_if_needed();
     }
 }
 
@@ -386,13 +344,8 @@ void do_statusbar_left(void)
 void do_statusbar_right(void)
 {
     if (statusbar_x < strlen(answer)) {
-	size_t pww_save = statusbar_pww;
-
 	statusbar_x = move_mbright(answer, statusbar_x);
-	statusbar_pww = statusbar_xplustabs();
-
-	if (need_statusbar_update(pww_save))
-	    update_statusbar_line(answer, statusbar_x);
+	update_bar_if_needed();
     }
 }
 
@@ -400,7 +353,7 @@ void do_statusbar_right(void)
 void do_statusbar_backspace(void)
 {
     if (statusbar_x > 0) {
-	do_statusbar_left();
+	statusbar_x = move_mbleft(answer, statusbar_x);
 	do_statusbar_delete();
     }
 }
@@ -411,19 +364,17 @@ void do_statusbar_delete(void)
     statusbar_pww = statusbar_xplustabs();
 
     if (answer[statusbar_x] != '\0') {
-	int char_buf_len = parse_mbchar(answer + statusbar_x, NULL,
-		NULL);
+	int char_buf_len = parse_mbchar(answer + statusbar_x, NULL, NULL);
 	size_t line_len = strlen(answer + statusbar_x);
 
 	assert(statusbar_x < strlen(answer));
 
-	charmove(answer + statusbar_x, answer + statusbar_x +
-		char_buf_len, strlen(answer) - statusbar_x -
-		char_buf_len + 1);
+	charmove(answer + statusbar_x, answer + statusbar_x + char_buf_len,
+			strlen(answer) - statusbar_x - char_buf_len + 1);
 
 	null_at(&answer, statusbar_x + line_len - char_buf_len);
 
-	update_statusbar_line(answer, statusbar_x);
+	update_the_statusbar();
     }
 }
 
@@ -443,168 +394,57 @@ void do_statusbar_cut_text(void)
 	statusbar_pww = statusbar_xplustabs();
     }
 
-    update_statusbar_line(answer, statusbar_x);
+    update_the_statusbar();
 }
 
 #ifndef NANO_TINY
-/* Move to the next word in the prompt text.  If allow_punct is TRUE,
- * treat punctuation as part of a word.  Return TRUE if we started on a
- * word, and FALSE otherwise. */
-bool do_statusbar_next_word(bool allow_punct)
+/* Move to the next word in the prompt text. */
+void do_statusbar_next_word(void)
 {
-    size_t pww_save = statusbar_pww;
-    char *char_mb;
-    int char_mb_len;
-    bool end_line = FALSE, started_on_word = FALSE;
+    bool seen_space = !is_word_mbchar(answer + statusbar_x, FALSE);
 
     assert(answer != NULL);
 
-    char_mb = charalloc(mb_cur_max());
+    /* Move forward until we reach the start of a word. */
+    while (answer[statusbar_x] != '\0') {
+	statusbar_x = move_mbright(answer, statusbar_x);
 
-    /* Move forward until we find the character after the last letter of
-     * the current word. */
-    while (!end_line) {
-	char_mb_len = parse_mbchar(answer + statusbar_x, char_mb, NULL);
-
-	/* If we've found it, stop moving forward through the current
-	 * line. */
-	if (!is_word_mbchar(char_mb, allow_punct))
+	/* If this is not a word character, then it's a separator; else
+	 * if we've already seen a separator, then it's a word start. */
+	if (!is_word_mbchar(answer + statusbar_x, FALSE))
+	    seen_space = TRUE;
+	else if (seen_space)
 	    break;
-
-	/* If we haven't found it, then we've started on a word, so set
-	 * started_on_word to TRUE. */
-	started_on_word = TRUE;
-
-	if (answer[statusbar_x] == '\0')
-	    end_line = TRUE;
-	else
-	    statusbar_x += char_mb_len;
     }
 
-    /* Move forward until we find the first letter of the next word. */
-    if (answer[statusbar_x] == '\0')
-	end_line = TRUE;
-    else
-	statusbar_x += char_mb_len;
-
-    while (!end_line) {
-	char_mb_len = parse_mbchar(answer + statusbar_x, char_mb, NULL);
-
-	/* If we've found it, stop moving forward through the current
-	 * line. */
-	if (is_word_mbchar(char_mb, allow_punct))
-	    break;
-
-	if (answer[statusbar_x] == '\0')
-	    end_line = TRUE;
-	else
-	    statusbar_x += char_mb_len;
-    }
-
-    free(char_mb);
-
-    statusbar_pww = statusbar_xplustabs();
-
-    if (need_statusbar_update(pww_save))
-	update_statusbar_line(answer, statusbar_x);
-
-    /* Return whether we started on a word. */
-    return started_on_word;
+    update_bar_if_needed();
 }
 
-/* Move to the previous word in the prompt text.  If allow_punct is
- * TRUE, treat punctuation as part of a word.  Return TRUE if we started
- * on a word, and FALSE otherwise. */
-bool do_statusbar_prev_word(bool allow_punct)
+/* Move to the previous word in the prompt text. */
+void do_statusbar_prev_word(void)
 {
-    size_t pww_save = statusbar_pww;
-    char *char_mb;
-    int char_mb_len;
-    bool begin_line = FALSE, started_on_word = FALSE;
+    bool seen_a_word = FALSE, step_forward = FALSE;
 
     assert(answer != NULL);
 
-    char_mb = charalloc(mb_cur_max());
-
-    /* Move backward until we find the character before the first letter
-     * of the current word. */
-    while (!begin_line) {
-	char_mb_len = parse_mbchar(answer + statusbar_x, char_mb, NULL);
-
-	/* If we've found it, stop moving backward through the current
-	 * line. */
-	if (!is_word_mbchar(char_mb, allow_punct))
-	    break;
-
-	/* If we haven't found it, then we've started on a word, so set
-	 * started_on_word to TRUE. */
-	started_on_word = TRUE;
-
-	if (statusbar_x == 0)
-	    begin_line = TRUE;
-	else
-	    statusbar_x = move_mbleft(answer, statusbar_x);
-    }
-
-    /* Move backward until we find the last letter of the previous
-     * word. */
-    if (statusbar_x == 0)
-	begin_line = TRUE;
-    else
+    /* Move backward until we pass over the start of a word. */
+    while (statusbar_x != 0) {
 	statusbar_x = move_mbleft(answer, statusbar_x);
 
-    while (!begin_line) {
-	char_mb_len = parse_mbchar(answer + statusbar_x, char_mb, NULL);
-
-	/* If we've found it, stop moving backward through the current
-	 * line. */
-	if (is_word_mbchar(char_mb, allow_punct))
+	if (is_word_mbchar(answer + statusbar_x, FALSE))
+	    seen_a_word = TRUE;
+	else if (seen_a_word) {
+	    /* This is space now: we've overshot the start of the word. */
+	    step_forward = TRUE;
 	    break;
-
-	if (statusbar_x == 0)
-	    begin_line = TRUE;
-	else
-	    statusbar_x = move_mbleft(answer, statusbar_x);
-    }
-
-    /* If we've found it, move backward until we find the character
-     * before the first letter of the previous word. */
-    if (!begin_line) {
-	if (statusbar_x == 0)
-	    begin_line = TRUE;
-	else
-	    statusbar_x = move_mbleft(answer, statusbar_x);
-
-	while (!begin_line) {
-	    char_mb_len = parse_mbchar(answer + statusbar_x, char_mb,
-		NULL);
-
-	    /* If we've found it, stop moving backward through the
-	     * current line. */
-	    if (!is_word_mbchar(char_mb, allow_punct))
-		break;
-
-	    if (statusbar_x == 0)
-		begin_line = TRUE;
-	    else
-		statusbar_x = move_mbleft(answer, statusbar_x);
 	}
-
-	/* If we've found it, move forward to the first letter of the
-	 * previous word. */
-	if (!begin_line)
-	    statusbar_x += char_mb_len;
     }
 
-    free(char_mb);
+    if (step_forward)
+	/* Move one character forward again to sit on the start of the word. */
+	statusbar_x = move_mbright(answer, statusbar_x);
 
-    statusbar_pww = statusbar_xplustabs();
-
-    if (need_statusbar_update(pww_save))
-	update_statusbar_line(answer, statusbar_x);
-
-    /* Return whether we started on a word. */
-    return started_on_word;
+    update_bar_if_needed();
 }
 #endif /* !NANO_TINY */
 
@@ -634,7 +474,6 @@ void do_statusbar_verbatim_input(bool *got_enter)
     free(output);
 }
 
-
 /* Return the placewewant associated with statusbar_x, i.e. the
  * zero-based column position of the cursor.  The value will be no
  * smaller than statusbar_x. */
@@ -667,18 +506,16 @@ void reset_statusbar_cursor(void)
 	get_statusbar_page_start(start_col, start_col + xpt));
 }
 
-/* Repaint the statusbar when getting a character in
- * get_prompt_string().  The statusbar text line will be displayed
- * starting with curranswer[index]. */
-void update_statusbar_line(const char *curranswer, size_t index)
+/* Repaint the statusbar. */
+void update_the_statusbar(void)
 {
-    size_t start_col, page_start;
+    size_t start_col, index, page_start;
     char *expanded;
 
-    assert(prompt != NULL && index <= strlen(curranswer));
+    assert(prompt != NULL && statusbar_x <= strlen(answer));
 
     start_col = strlenpt(prompt) + 2;
-    index = strnlenpt(curranswer, index);
+    index = strnlenpt(answer, statusbar_x);
     page_start = get_statusbar_page_start(start_col, start_col + index);
 
     if (interface_color_pair[TITLE_BAR].bright)
@@ -691,39 +528,32 @@ void update_statusbar_line(const char *curranswer, size_t index)
     waddch(bottomwin, ':');
     waddch(bottomwin, (page_start == 0) ? ' ' : '$');
 
-    expanded = display_string(curranswer, page_start, COLS - start_col -
-	1, FALSE);
+    expanded = display_string(answer, page_start, COLS - start_col - 1, FALSE);
     waddstr(bottomwin, expanded);
     free(expanded);
 
     wattroff(bottomwin, A_BOLD);
     wattroff(bottomwin, interface_color_pair[TITLE_BAR].pairnum);
+
     statusbar_pww = statusbar_xplustabs();
     reset_statusbar_cursor();
     wnoutrefresh(bottomwin);
 }
 
-/* Return TRUE if we need an update after moving the cursor, and FALSE
- * otherwise.  We need an update if pww_save and statusbar_pww are on
- * different pages. */
-bool need_statusbar_update(size_t pww_save)
+/* Update the statusbar line /if/ the placewewant changes page. */
+void update_bar_if_needed(void)
 {
     size_t start_col = strlenpt(prompt) + 2;
+    size_t was_pww = statusbar_pww;
 
-    return get_statusbar_page_start(start_col, start_col + pww_save) !=
-	get_statusbar_page_start(start_col, start_col + statusbar_pww);
+    statusbar_pww = statusbar_xplustabs();
+
+    if (get_statusbar_page_start(start_col, start_col + statusbar_pww) !=
+		get_statusbar_page_start(start_col, start_col + was_pww))
+	update_the_statusbar();
 }
 
-/* Unconditionally redraw the entire screen, and then refresh it using
- * refresh_func(). */
-void total_statusbar_refresh(void (*refresh_func)(void))
-{
-    total_redraw();
-    refresh_func();
-}
-
-/* Get a string of input at the statusbar prompt.  This should only be
- * called from do_prompt(). */
+/* Get a string of input at the statusbar prompt. */
 functionptrtype get_prompt_string(int *actual, bool allow_tabs,
 #ifndef DISABLE_TABCOMP
 	bool allow_files,
@@ -737,7 +567,6 @@ functionptrtype get_prompt_string(int *actual, bool allow_tabs,
 {
     int kbinput = ERR;
     bool ran_func, finished;
-    size_t curranswer_len;
     functionptrtype func;
 #ifndef DISABLE_TABCOMP
     bool tabbed = FALSE;
@@ -759,23 +588,9 @@ functionptrtype get_prompt_string(int *actual, bool allow_tabs,
 #endif /* !DISABLE_HISTORIES */
 
     answer = mallocstrcpy(answer, curranswer);
-    curranswer_len = strlen(answer);
 
-    /* If reset_statusbar_x is TRUE, restore statusbar_x and
-     * statusbar_pww to what they were before this prompt.  Then, if
-     * statusbar_x is uninitialized or past the end of curranswer, put
-     * statusbar_x at the end of the string and update statusbar_pww
-     * based on it.  We do these things so that the cursor position
-     * stays at the right place if a prompt-changing toggle is pressed,
-     * or if this prompt was started from another prompt and we cancel
-     * out of it. */
-    if (reset_statusbar_x) {
-	statusbar_x = old_statusbar_x;
-	statusbar_pww = old_pww;
-    }
-
-    if (statusbar_x == (size_t)-1 || statusbar_x > curranswer_len) {
-	statusbar_x = curranswer_len;
+    if (statusbar_x > strlen(answer)) {
+	statusbar_x = strlen(answer);
 	statusbar_pww = statusbar_xplustabs();
     }
 
@@ -783,10 +598,9 @@ functionptrtype get_prompt_string(int *actual, bool allow_tabs,
     fprintf(stderr, "get_prompt_string: answer = \"%s\", statusbar_x = %lu\n", answer, (unsigned long) statusbar_x);
 #endif
 
-    update_statusbar_line(answer, statusbar_x);
+    update_the_statusbar();
 
-    /* Refresh the edit window and the statusbar before getting
-     * input. */
+    /* Refresh edit window and statusbar before getting input. */
     wnoutrefresh(edit);
     wnoutrefresh(bottomwin);
 
@@ -800,13 +614,12 @@ functionptrtype get_prompt_string(int *actual, bool allow_tabs,
 	assert(statusbar_x <= strlen(answer));
 
 #ifndef NANO_TINY
-    if (kbinput == KEY_WINCH) {
-	refresh_func();
-	update_statusbar_line(answer, statusbar_x);
-	continue;
-    }
+	if (kbinput == KEY_WINCH) {
+	    refresh_func();
+	    update_the_statusbar();
+	    continue;
+	}
 #endif
-
 	func = func_from_key(&kbinput);
 
 	if (func == do_cancel || func == do_enter)
@@ -834,7 +647,7 @@ functionptrtype get_prompt_string(int *actual, bool allow_tabs,
 		answer = input_tab(answer, allow_files, &statusbar_x,
 				   &tabbed, refresh_func, list);
 
-	    update_statusbar_line(answer, statusbar_x);
+	    update_the_statusbar();
 	} else
 #endif /* !DISABLE_TABCOMP */
 #ifndef DISABLE_HISTORIES
@@ -852,7 +665,7 @@ functionptrtype get_prompt_string(int *actual, bool allow_tabs,
 		    statusbar_x = strlen(answer);
 		}
 
-		update_statusbar_line(answer, statusbar_x);
+		update_the_statusbar();
 
 		/* This key has a shortcut-list entry when it's used to
 		 * move to an older search, which means that finished has
@@ -873,12 +686,12 @@ functionptrtype get_prompt_string(int *actual, bool allow_tabs,
 		 * history list, answer is blank, and magichistory is set,
 		 * save magichistory in answer. */
 		if ((*history_list)->next == NULL &&
-		    *answer == '\0' && magichistory != NULL) {
-			answer = mallocstrcpy(answer, magichistory);
-			statusbar_x = strlen(answer);
-		    }
+			*answer == '\0' && magichistory != NULL) {
+		    answer = mallocstrcpy(answer, magichistory);
+		    statusbar_x = strlen(answer);
+		}
 
-		update_statusbar_line(answer, statusbar_x);
+		update_the_statusbar();
 
 		/* This key has a shortcut-list entry when it's used to
 		 * move to a newer search, which means that finished has
@@ -889,7 +702,7 @@ functionptrtype get_prompt_string(int *actual, bool allow_tabs,
 	} else
 #endif /* !DISABLE_HISTORIES */
 	if (func == do_help_void) {
-	    update_statusbar_line(answer, statusbar_x);
+	    update_the_statusbar();
 
 	    /* This key has a shortcut-list entry when it's used to go to
 	     * the help browser or display a message indicating that help
@@ -907,37 +720,23 @@ functionptrtype get_prompt_string(int *actual, bool allow_tabs,
 #if !defined(DISABLE_HISTORIES) && !defined(DISABLE_TABCOMP)
 	last_kbinput = kbinput;
 #endif
-
 	reset_statusbar_cursor();
 	wnoutrefresh(bottomwin);
     }
 
 #ifndef DISABLE_HISTORIES
-    /* Set the current position in the history list to the bottom,
-     * and free magichistory if we need to. */
+    /* Set the current position in the history list to the bottom. */
     if (history_list != NULL) {
 	history_reset(*history_list);
-
 	free(magichistory);
     }
 #endif
 
-    /* We've finished putting in an answer or run a normal shortcut's
-     * associated function, so reset statusbar_x and statusbar_pww.  If
-     * we've finished putting in an answer, reset the statusbar cursor
-     * position too. */
-    if (func) {
-	if (func == do_cancel || func == do_enter || ran_func) {
-	    statusbar_x = old_statusbar_x;
-	    statusbar_pww = old_pww;
-
-	    if (!ran_func)
-		reset_statusbar_x = TRUE;
-    /* Otherwise, we're still putting in an answer or a shortcut with
-     * an associated function, so leave the statusbar cursor position
-     * alone. */
-	} else
-	    reset_statusbar_x = FALSE;
+    /* If we're done with this prompt, restore the cursor position
+     * to what it was at the /previous/ prompt, in case there was. */
+    if (func == do_cancel || func == do_enter) {
+	statusbar_x = old_statusbar_x;
+	statusbar_pww = old_pww;
     }
 
     *actual = kbinput;
@@ -955,8 +754,7 @@ functionptrtype get_prompt_string(int *actual, bool allow_tabs,
  *
  * The allow_tabs parameter indicates whether we should allow tabs to be
  * interpreted.  The allow_files parameter indicates whether we should
- * allow all files (as opposed to just directories) to be tab
- * completed. */
+ * allow all files (as opposed to just directories) to be tab completed. */
 int do_prompt(bool allow_tabs,
 #ifndef DISABLE_TABCOMP
 	bool allow_files,
@@ -974,10 +772,6 @@ int do_prompt(bool allow_tabs,
     bool list = FALSE;
 #endif
 
-    /* prompt has been freed and set to NULL unless the user resized
-     * while at the statusbar prompt. */
-    free(prompt);
-
     prompt = charalloc(((COLS - 4) * mb_cur_max()) + 1);
 
     bottombars(menu);
@@ -1031,23 +825,12 @@ int do_prompt(bool allow_tabs,
     return retval;
 }
 
-/* This function forces a reset of the statusbar cursor position.  It
- * should be called when we get out of all statusbar prompts. */
-void do_prompt_abort(void)
-{
-    /* Uninitialize the old cursor position in answer. */
-    old_statusbar_x = (size_t)-1;
-    old_pww = (size_t)-1;
-
-    reset_statusbar_x = TRUE;
-}
-
 /* Ask a simple Yes/No (and optionally All) question, specified in msg,
  * on the statusbar.  Return 1 for Yes, 0 for No, 2 for All (if all is
  * TRUE when passed in), and -1 for Cancel. */
 int do_yesno_prompt(bool all, const char *msg)
 {
-    int ok = -2, width = 16;
+    int response = -2, width = 16;
     const char *yesstr;		/* String of Yes characters accepted. */
     const char *nostr;		/* Same for No. */
     const char *allstr;		/* And All, surprise! */
@@ -1127,52 +910,43 @@ int do_yesno_prompt(bool all, const char *msg)
 	func = func_from_key(&kbinput);
 
 	if (func == do_cancel)
-	    ok = -1;
+	    response = -1;
 #ifndef DISABLE_MOUSE
 	else if (kbinput == KEY_MOUSE) {
-		/* We can click on the Yes/No/All shortcut list to
-		 * select an answer. */
-		if (get_mouseinput(&mouse_x, &mouse_y, FALSE) == 0 &&
-			wmouse_trafo(bottomwin, &mouse_y, &mouse_x,
-			FALSE) && mouse_x < (width * 2) &&
-			mouse_y > 0) {
-		    int x = mouse_x / width;
-			/* Calculate the x-coordinate relative to the
-			 * two columns of the Yes/No/All shortcuts in
-			 * bottomwin. */
-		    int y = mouse_y - 1;
-			/* Calculate the y-coordinate relative to the
-			 * beginning of the Yes/No/All shortcuts in
-			 * bottomwin, i.e. with the sizes of topwin,
-			 * edit, and the first line of bottomwin
-			 * subtracted out. */
+	    /* We can click on the Yes/No/All shortcuts to select an answer. */
+	    if (get_mouseinput(&mouse_x, &mouse_y, FALSE) == 0 &&
+			wmouse_trafo(bottomwin, &mouse_y, &mouse_x, FALSE) &&
+			mouse_x < (width * 2) && mouse_y > 0) {
+		int x = mouse_x / width;
+			/* The x-coordinate among the Yes/No/All shortcuts. */
+		int y = mouse_y - 1;
+			/* The y-coordinate among the Yes/No/All shortcuts. */
 
-		    assert(0 <= x && x <= 1 && 0 <= y && y <= 1);
+		assert(0 <= x && x <= 1 && 0 <= y && y <= 1);
 
-		    /* x == 0 means they clicked Yes or No.  y == 0
-		     * means Yes or All. */
-		    ok = -2 * x * y + x - y + 1;
+		/* x == 0 means they clicked Yes or No.
+		 * y == 0 means Yes or All. */
+		response = -2 * x * y + x - y + 1;
 
-		    if (ok == 2 && !all)
-			ok = -2;
-		}
+		if (response == 2 && !all)
+		    response = -2;
+	    }
 	}
 #endif /* !DISABLE_MOUSE */
 	else if (func == total_refresh) {
 	    total_redraw();
 	    continue;
 	} else {
-		/* Look for the kbinput in the Yes, No and (optionally)
-		 * All strings. */
-		if (strchr(yesstr, kbinput) != NULL)
-		    ok = 1;
-		else if (strchr(nostr, kbinput) != NULL)
-		    ok = 0;
-		else if (all && strchr(allstr, kbinput) != NULL)
-		    ok = 2;
+	    /* Look for the kbinput in the Yes, No (and All) strings. */
+	    if (strchr(yesstr, kbinput) != NULL)
+		response = 1;
+	    else if (strchr(nostr, kbinput) != NULL)
+		response = 0;
+	    else if (all && strchr(allstr, kbinput) != NULL)
+		response = 2;
 	}
-    } while (ok == -2);
+    } while (response == -2);
 
     currmenu = oldmenu;
-    return ok;
+    return response;
 }
