@@ -1,4 +1,4 @@
-/* $Id: proto.h 5532 2016-01-04 10:37:11Z bens $ */
+/* $Id: proto.h 5612 2016-02-06 11:44:33Z bens $ */
 /**************************************************************************
  *   proto.h                                                              *
  *                                                                        *
@@ -128,7 +128,7 @@ extern filestruct *searchbot;
 extern filestruct *replace_history;
 extern filestruct *replaceage;
 extern filestruct *replacebot;
-extern poshiststruct *poshistory;
+extern poshiststruct *position_history;
 #endif
 
 #ifdef HAVE_REGEX_H
@@ -280,7 +280,7 @@ void do_cut_till_eof(void);
 #endif
 void do_uncut_text(void);
 
-/* All functions in files.c. */
+/* Most functions in files.c. */
 void make_new_buffer(void);
 void initialize_buffer_text(void);
 bool open_buffer(const char *filename, bool undoable);
@@ -527,15 +527,15 @@ void do_statusbar_backspace(void);
 void do_statusbar_delete(void);
 void do_statusbar_cut_text(void);
 #ifndef NANO_TINY
-bool do_statusbar_next_word(bool allow_punct);
-bool do_statusbar_prev_word(bool allow_punct);
+void do_statusbar_next_word(void);
+void do_statusbar_prev_word(void);
 #endif
 void do_statusbar_verbatim_input(bool *got_enter);
 size_t statusbar_xplustabs(void);
 size_t get_statusbar_page_start(size_t start_col, size_t column);
 void reset_statusbar_cursor(void);
-void update_statusbar_line(const char *curranswer, size_t index);
-bool need_statusbar_update(size_t pww_save);
+void update_the_statusbar(void);
+void update_bar_if_needed(void);
 void total_statusbar_refresh(void (*refresh_func)(void));
 functionptrtype get_prompt_string(int *value, bool allow_tabs,
 #ifndef DISABLE_TABCOMP
@@ -748,7 +748,7 @@ void new_magicline(void);
 void remove_magicline(void);
 void mark_order(const filestruct **top, size_t *top_x, const filestruct
 	**bot, size_t *bot_x, bool *right_side_up);
-void discard_until(undo *thisone);
+void discard_until(const undo *thisitem, openfilestruct *thefile);
 void add_undo(undo_type action);
 void update_undo(undo_type action);
 #endif
@@ -822,8 +822,6 @@ void do_credits(void);
 
 /* May as well throw these here, since they are just placeholders. */
 void do_cancel(void);
-void do_page_up(void);
-void do_page_down(void);
 void case_sens_void(void);
 void regexp_void(void);
 void gototext_void(void);
