@@ -1,4 +1,4 @@
-/* $Id: rcfile.c 5349 2015-08-09 16:31:01Z bens $ */
+/* $Id: rcfile.c 5625 2016-02-10 15:06:45Z bens $ */
 /**************************************************************************
  *   rcfile.c                                                             *
  *                                                                        *
@@ -53,7 +53,6 @@ static const rcoption rcopts[] = {
 #ifndef DISABLE_MULTIBUFFER
     {"multibuffer", MULTIBUFFER},
 #endif
-    {"nofollow", NOFOLLOW_SYMLINKS},
     {"nohelp", NO_HELP},
     {"nonewlines", NO_NEWLINES},
 #ifndef DISABLE_WRAPPING
@@ -389,20 +388,6 @@ void parse_syntax(char *ptr)
 }
 #endif /* !DISABLE_COLOR */
 
-int check_bad_binding(sc *s)
-{
-#define BADLISTLEN 1
-    key_type badtypes[BADLISTLEN] = {META};
-    int badseqs[BADLISTLEN] = { 91 };
-    int i;
-
-    for (i = 0; i < BADLISTLEN; i++)
-	if (s->type == badtypes[i] && s->seq == badseqs[i])
-	    return 1;
-
-    return 0;
-}
-
 /* Check whether the given executable function is "universal" (meaning
  * any horizontal movement or deletion) and thus is present in almost
  * all menus. */
@@ -467,6 +452,9 @@ void parse_binding(char *ptr, bool dobind)
     else if (keycopy[0] != '^' && keycopy[0] != 'M' && keycopy[0] != 'F') {
 	rcfile_error(N_("Key name must begin with \"^\", \"M\", or \"F\""));
 	goto free_copy;
+    } else if (keycopy[0] == '^' && (keycopy[1] < 64 || keycopy[1] > 127)) {
+	rcfile_error(N_("Key name %s is invalid"), keycopy);
+	goto free_copy;
     }
 
     if (dobind) {
@@ -539,16 +527,17 @@ void parse_binding(char *ptr, bool dobind)
 	newsc->menus = menu;
 	newsc->type = strtokeytype(newsc->keystr);
 	assign_keyinfo(newsc);
-#ifdef DEBUG
-	fprintf(stderr, "s->keystr = \"%s\"\n", newsc->keystr);
-	fprintf(stderr, "s->seq = \"%d\"\n", newsc->seq);
-#endif
 
-	if (check_bad_binding(newsc)) {
+	/* Do not allow rebinding the equivalent of the Escape key. */
+	if (newsc->type == META && newsc->seq == 91) {
 	    rcfile_error(N_("Sorry, keystroke \"%s\" may not be rebound"), newsc->keystr);
 	    free(newsc);
 	    goto free_copy;
 	}
+#ifdef DEBUG
+	fprintf(stderr, "s->keystr = \"%s\"\n", newsc->keystr);
+	fprintf(stderr, "s->seq = \"%d\"\n", newsc->seq);
+#endif
     }
 
     /* Now find and delete any existing same shortcut in the menu(s). */
